package net.sf.appinfo;

import net.sf.appinfo.exception.InitializationException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Abstract ApplicationInfo implementation with base mechanisms related to initlialization.
 *
 * Class should be extended by every ApplicationInfo implementation with initilization.
 *
 * @since 0.4.0
 *
 * @author Marcin Zajączkowski, 2010-10-03
 */
public abstract class AbstractInitAwareApplicationInfo implements InitAwareApplicationInfo {

    protected final Logger log = LoggerFactory.getLogger(getClass());

    private String name = DEFAULT_NOT_INITED;
    private String version = DEFAULT_NOT_INITED;
    private String buildNumber = DEFAULT_NOT_INITED;
    private String buildDate = DEFAULT_NOT_INITED;

    private boolean inited = false;

    /** {@inheritDoc} */
    public boolean isInited() {
        return inited;
    }

    /** {@inheritDoc} */
    public final void initInfo() throws InitializationException {
        if (isInited()) {
            log.info("ApplicationInfo already initialized. Do nothing.");
            return;
        }
        callInternalInitAndSetInitedFlagWhenSuccessful();
    }

    private void callInternalInitAndSetInitedFlagWhenSuccessful() throws InitializationException {
        initClass();
        inited = true;
    }

    /**
     * Performs internal class initialization.
     *
     * Has to be implemented in every subclass.
     *
     * @throws InitializationException problem with initialization
     */
    protected abstract void initClass() throws InitializationException;

    /**
     * Initializes applicationInfo in a safe way - no exception is thrown.
     *
     * Should be used for initialization in stand alone mode - e.g. from IoC contener. Initialization from other
     * ApplicationInfo classes should use initInfo().
     */
    public void standaloneSafeInit() {
        try {
            initInfo();
//CHECKSTYLE:OFF - that method is intented to be safe in call - all exceptions (except Throwable) should be caught
        } catch (Exception e) {
//CHECKSTYLE:ON
            printErrorWithStackTrace("Initialization Error. Object was not initialized.", e);
        }
    }

    /**
     * Artificial construction logging given error message and stacktrace.
     *
     * Created to prevent stack trace in log during tests.
     *
     * Thanks for Jarek Woloch for a suggestion. 
     *
     * @param errorMessage error message
     * @param exception exception
     */
    protected void printErrorWithStackTrace(String errorMessage, Exception exception) {
        log.error(errorMessage, exception);
    }

    public String getName() {
        return name;
    }

    public String getVersion() {
        return version;
    }

    public String getBuildNumber() {
        return buildNumber;
    }

    public String getBuildDate() {
        return buildDate;
    }

    protected void setName(String name) {
        this.name = name;
    }

    protected void setVersion(String version) {
        this.version = version;
    }

    protected void setBuildNumber(String buildNumber) {
        this.buildNumber = buildNumber;
    }

    protected void setBuildDate(String buildDate) {
        this.buildDate = buildDate;
    }
}
