/* Copyright (c) 2010 Marcin Zajaczkowski
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.appinfo;

import net.sf.appinfo.exception.InitializationException;
import net.sf.appinfo.reader.ManifestReader;
import org.springframework.util.Assert;

import java.io.IOException;
import java.util.jar.Attributes;
import java.util.jar.Manifest;

import static org.springframework.util.StringUtils.hasText;

/**
 * Simple ApplicationInfo implementation reading data from manifest.
 *
 * It's the simplier version of too much complicated ManifestApplicationInfonet. Fallback mechanism was moved to
 * separate class ChainedApplicationInfo. Data from mannifest is provided by ManifestReader.
 *
 * @see net.sf.appinfo.ChainedApplicationInfo
 * @see net.sf.appinfo.reader.ManifestReader
 *
 * @since 0.4.0
 *
 * @author Marcin Zajączkowski, 2010-10-03
 */
public class SimplierManifestApplicationInfo extends AbstractInitAwareApplicationInfo {

    private static final String NAME_ENTRY_CONSTANT = "Specification-Title";
    private static final String VERSION_ENTRY_CONSTANT = "Specification-Version";
    private static final String BUILD_NUMBER_ENTRY_CONSTANT = "Build-Number";
    private static final String BUILD_DATE_ENTRY_CONSTANT = "Build-Date";

    private final ManifestReader manifestReader;

    public SimplierManifestApplicationInfo(ManifestReader manifestReader) {
        super();
        Assert.notNull(manifestReader, "manifestReader cannot be null");
        this.manifestReader = manifestReader;
    }

    @Override
    public void initClass() throws InitializationException {
        try {
            setClassFieldsUsingManifest();
//CHECKSTYLE:OFF - exceptions should be converted to InitializationException to be caught by calling class
        } catch (Exception e) {
//CHECKSTYLE:ON
            //when reading manifest other then IOException exceptions could be also thrown (like NPE)
            //and should be served as InitializationException
            throw new InitializationException(e);
        }
    }

    private void setClassFieldsUsingManifest() throws IOException {
        Manifest manifest = manifestReader.getManifest();
        Attributes attributes = manifest.getMainAttributes();
        setName(getValueFromMapOrDefaultValueIfAbsentOrEmpty(attributes, NAME_ENTRY_CONSTANT, DEFAULT_NAME));
        setVersion(getValueFromMapOrDefaultValueIfAbsentOrEmpty(attributes, VERSION_ENTRY_CONSTANT, DEFAULT_VERSION));
        setBuildNumber(getValueFromMapOrDefaultValueIfAbsentOrEmpty(attributes,
                BUILD_NUMBER_ENTRY_CONSTANT, DEFAULT_BUILD_NUMBER));
        setBuildDate(getValueFromMapOrDefaultValueIfAbsentOrEmpty(attributes,
                BUILD_DATE_ENTRY_CONSTANT, DEFAULT_BUILD_DATE));
    }

    private String getValueFromMapOrDefaultValueIfAbsentOrEmpty(Attributes attributes, String entryName,
                                                                String defaultValue) {
        String value = attributes.getValue(entryName);
        if (hasText(value)) {
            return value;
        } else {
            log.debug("Value for entry {} set to default {}", entryName, defaultValue);
            return defaultValue;
        }
    }
}
